
/*****There are 64 TCA7313 Address combinations*****
 *  		0x20-0x5E or 0xA0-0xDE                 *
 ***************************************************/

/****************************************************
 * 				Read Input Registers                *
 * 	 PORT0: GPIO07-GPIO00    PORT1: GPIO17-GPIO10   *
 ****************************************************/
#define INPUT_PORT0			0x00
#define INPUT_PORT1			0x01

/****************************************************
 * 			    Blink Phase 0 Registers             *
 * PHASE0_0: GPIO07-GPIO00  PHASE0_1: GPIO17-GPIO10 *
 ****************************************************/
#define BLINK_PHASE0_0		0x02
#define BLINK_PHASE0_1		0x03

/****************************************************
 * 	        Ports Configuration Registers           *
 *   PORT0: GPIO07-GPIO00     PORT1: GPIO17-GPIO10  *
 *   bitX set to 1 pin configured as input          *
 *   bitX set to 0 pin configured as output         *
 ****************************************************/
#define CONFG_PORT0			0x06
#define CONFG_PORT1			0x07

/****************************************************
 * 			    Blink Phase 1 Register              *
 * PHASE1_0: GPIO07-GPIO00  PHASE1_1: GPIO17-GPIO10 *
 ****************************************************/
#define BLINK_PHASE1_0		0x0A
#define BLINK_PHASE1_1 		0x0B

/****************************************************
 * 	    Master Multipurpose intensity register      *
 * B7-B4: Master Intensity   B3-B0: Multi intensity *
 ****************************************************/
#define MASTER_INTEN		0x0E

/****************************************************
 * 			    Configuration Registers             *
 ****************************************************/
#define CONFG				0x0F
#define 	INT_STATUS 	0x80				//Used to mask interrupt status
#define 	MULT_HIGH_Z 0x10				//Multipurpose pin set to HIGH_Z
#define		BLINK		0x01				//Enable BLINK
#define		BLINK_FLIP  0x02				//Set to 0: Blink Phase 0 used; Set to 1: Blink Phase 1 used
#define 	GLOBAL_INTEN 0x04				//Global intensity enabled
#define		INTERRUPT   0x08				//Enable multipurpose pin as /INT
#define		MULT_PHASE1 0x20

/****************************************************
 *      Individual Output intensity registers       *
 *      PWM Set to X/16 duty cycle					*
 *      0: 1/16 	     	 8: 9/16				*
 *      1: 2/16 			 9: 10/16				*
 *      2: 3/16 			10: 11/16				*
 *      3: 4/16 			11: 12/16				*
 *      4: 5/16				12: 13/16 				*
 *      5: 6/16				13: 14/16				*
 *      6: 7/16				14: 15/16 				*
 *      7: 8/16				15: 16/16 				*
 ****************************************************/
#define	OUTPUT_INTEN1		0x10
#define	OUTPUT_INTEN3		0x11
#define	OUTPUT_INTEN5		0x12
#define	OUTPUT_INTEN7		0x13
#define	OUTPUT_INTEN11		0x14
#define	OUTPUT_INTEN13		0x15
#define	OUTPUT_INTEN15		0x16
#define	OUTPUT_INTEN17		0x17
#define 	MASK_EVEN		0x0F
#define		MASK_ODD		0xF0

