#include "msp430g2553.h"
#include "USCI_I2C.h"
//#include "boosterPack.h"

#define I2C_FLAG 0x01
#define BLOCK_SIZE 63
#define NUM_BLOCKS 7

#define UART_RESET_FLAG 			0x80
#define UART_READ_OTP_FLAG 			0x08
#define UART_READ_REPORT_FLAG		0x40			//GET REport Command
#define UART_READ_INPUT_REPORT_FLAG	0x44			//Read Report
#define UART_WAKE_FLAG				0x20
#define UART_SLEEP_FLAG				0x10
#define UART_LED_FLAG 				0x04
#define UART_I2C_ADDRESS_FLAG  	 	0x02
#define UART_OTP_DATA_INC 			0x01
#define UART_OTP_WRITE_SETUP 		0x11


#define RESET 				0x02
#define READ_OTP 			0x04
#define READ_REPORT			0x08
#define WAKE				0x10
#define SLEEP				0x20
#define LED 				0x40
#define ADDRESS				0x80
#define READ_INPUT_REPORT 	0x100


unsigned const char arrayb[] = {0x91,0x1E,2,3,4,5,6,7,8,9,10,11,12,13,14,15,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15};
unsigned short counter = 0;
unsigned char block_count=0;
unsigned short byte_count=0;
unsigned char first=0;

#define FLASH_PTR 0xFC00
char *ptr;
unsigned char input_buffer[64];									//UART input buffer

unsigned char input_reg_address[5];
unsigned char input_Buffer_Counter = 0;							//Index for input_buffer[]
unsigned char UART_reg_count = 0;
unsigned short UART_data_byte_count = 0;								//Bytes to be read by UART
unsigned char FLAG = 1;											//Waiting for flag detection on UART
unsigned char I2C_ADDRESS = 0x3B;
unsigned char I2C_output[11];
unsigned short wake_Flag=0;										//Byte tells CPU what to handle after wake
unsigned short RAM_ADDRESS = 0xA000;

extern unsigned char NACK;										//NACK detected if equal to 1; I2C failed

unsigned short flash_ptr_offset = 0;

unsigned char array[45];
unsigned char dat1=0;

short OTP_count		= 0;
short OTP_count1	= 0;
unsigned short OTP_start_add=0;

void write_UART_byte(unsigned char byte);


void CLOCK_INIT(void);
void PORT_INIT(void);
void UART_I2C_INIT(void);
void FLASH_INIT(void);

void run_Booster_Demo(void);
void print_UART(char* string);
//void write_To_Flash(unsigned char, unsigned short);
void testfunction(char *);

void main(void)
{
	WDTCTL = WDTPW + WDTHOLD;                 						// Stop WDT

	CLOCK_INIT();													// Configure clock to 8 MHz
	PORT_INIT();													// Configure ports
	UART_I2C_INIT();												// Configure I2C
	FLASH_INIT();
	//IFG1 &= ~WDTIFG;												// Clear WDT flag
	//IE1 |= WDTIE;													// Enable WDT interrupt

	__delay_cycles(10000);											// Allow I2C devices time to power up

	int i;

	P2OUT &= ~BIT0;
	P1OUT |= BIT4;
	P1OUT &= ~BIT5;
	P1DIR |= BIT5;															//LS On/OFF enable; Active high
	P1DIR |= BIT4;															//LDO_EN; Active Low
	P2DIR |= BIT0;															//DC Boost Enable; Active high
	P2DIR &= ~BIT1;															//Interrupt pin

	while(1)
	{
		// Enter LPM0, interrupts enabled
		_bis_SR_register(LPM0_bits + GIE);
		if(wake_Flag & ADDRESS)
		{
			I2C_ADDRESS = (input_reg_address[1] >> 1);
			wake_Flag &= ~ADDRESS;
		}

		if(wake_Flag & 0x01)
		{
			I2C_output[0] = 0x00;
			I2C_output[1] = 0x01;
			I2C_Write_2command(2, I2C_ADDRESS, 0x0600, I2C_output, 0); 				//Reset Device
			if (NACK == 0x01) {
				write_UART_byte(0xF0);
				NACK = 0x00;
			}
			else {



				__no_operation();
				__no_operation();
				__no_operation();
				__no_operation();
				__no_operation();
				__no_operation();
				__no_operation();
				__no_operation();
				__no_operation();
				__no_operation();

				while (P2IN & BIT1);
				I2C_Read_no_reg(11, I2C_ADDRESS, I2C_output);							//Read Input Report to re
				P1OUT &= ~BIT4;														//turn on LDO
				P2OUT |= BIT0;														//turn on booster
				__delay_cycles(1000000);												//Delay for Stable Supply

				unsigned char* OTP_PTR = (unsigned char*)FLASH_PTR;
				while (OTP_count > 0) {
					if (OTP_count > 63) {
						I2C_Write_2command(64, I2C_ADDRESS, RAM_ADDRESS, OTP_PTR, 0);

					}
					else {
						I2C_Write_2command(OTP_count, I2C_ADDRESS, RAM_ADDRESS, OTP_PTR, 0);
					}
					OTP_count -= 64;
					RAM_ADDRESS += 64;
					OTP_PTR += 64;
				}

				I2C_output[0] = 0x80;
				I2C_Write_2command(1, I2C_ADDRESS, 0xC5C5, I2C_output, 0);			//0xC5C5 = 0x80
				I2C_Write_2command(1, I2C_ADDRESS, 0xC5C7, I2C_output, 0);			//0xC5C7 = 0x80

				I2C_output[0] = (OTP_start_add & 0xFF);
				I2C_output[1] = (OTP_start_add >> 8);
				I2C_Write_2command(2, I2C_ADDRESS, 0x0600, I2C_output, 0);			//Sends start Address format 0x0XXX

				I2C_output[0] = 0x06;
				I2C_Write_2command(1, I2C_ADDRESS, 0xC5C6, I2C_output, 0);			//0xC5C6 = 0X06
				I2C_output[0] = 0xCA;

				I2C_Write_2command(1, I2C_ADDRESS, 0xC5C7, I2C_output, 0);			//0xC5C7 = 0XCA

				__no_operation();
				__no_operation();
				__no_operation();
				__no_operation();
				__no_operation();
				__no_operation();
				__no_operation();
				__no_operation();
				__no_operation();
				__no_operation();
				__no_operation();
				__no_operation();
				while (!(P2IN & BIT1));												//WAIT FOR INT TO GO HIGH
				I2C_output[0] = 0x0C;
				I2C_Write_2command(1, I2C_ADDRESS, 0xC5C7, I2C_output, 0);			//ADDRESS 0xC5C7 = 0x0C


				for (i = 0; i< OTP_count1; i++)										//Loop this sequence for the number of bytes to be programmed
				{
					I2C_output[0] = 0x0E;
					I2C_output[1] = 0x0F;
					I2C_output[2] = 0x0E;
					I2C_Write_2command(3, I2C_ADDRESS, 0xC5C7, I2C_output, 0);		//Send 0x0E, 0x0F, 0x0E	to Address 0xC5C7

					P1OUT |= BIT5;													//turn on 7 V
					__delay_cycles(1600);											// wait 100 us
					P1OUT &= ~BIT5;													// turn off 7 V

					I2C_output[0] = 0x1A;
					I2C_Write_2command(1, I2C_ADDRESS, 0xC5C7, I2C_output, 0);		//Send 0x0A		to Address 0xC5C7
				}

				I2C_output[0] = 0x2A;
				I2C_Write_2command(1, I2C_ADDRESS, 0xC5C7, I2C_output, 0);			//0xC5C7 = 0x2A  End of OTP programming
				__delay_cycles(8000000);

				P1OUT |= BIT4;														//turn off LDO
				P2OUT &= ~BIT0;														//turn off booster
			}
			wake_Flag &= ~0x01;
			OTP_count1	= 0;
			OTP_count		= 0;
			byte_count=0;
			write_UART_byte(0xA0);

		}
		if(wake_Flag & LED)
		{
			I2C_output[0] = 0x20;
			I2C_output[1] = 0x03;
			I2C_output[2] = 0x00;
			I2C_output[3] = 0x07;
			I2C_output[4] = 0x01;
			I2C_output[5] = 0x00;
			I2C_output[6] = input_reg_address[1];

			I2C_Write_2command(7, I2C_ADDRESS, 0x0600, I2C_output, 0);
			if (NACK == 0x01) {
				write_UART_byte(0xF0);
				NACK = 0x00;
			}
			wake_Flag &= ~LED;
		}
		if(wake_Flag & WAKE)
		{
			I2C_output[0] = 0x00;
			I2C_output[1] = 0x08;
			I2C_Write_2command(2, I2C_ADDRESS, 0x0600, I2C_output, 0);
			if (NACK == 0x01) {
				write_UART_byte(0xF0);
				NACK = 0x00;
			}
			wake_Flag &= ~WAKE;
		}
		if(wake_Flag & SLEEP)
		{
			I2C_output[0] = 0x01;
			I2C_output[1] = 0x08;
			I2C_Write_2command(2, I2C_ADDRESS, 0x0600, I2C_output, 0);
			if (NACK == 0x01) {
				write_UART_byte(0xF0);
				NACK = 0x00;
			}
			wake_Flag &= ~SLEEP;
		}
		if(wake_Flag & RESET)
		{
			I2C_output[0] = 0x00;
			I2C_output[1] = 0x01;
			I2C_Write_2command(2, I2C_ADDRESS, 0x0600, I2C_output, 0); 				//Reset Device
			if (NACK == 0x01) {
				write_UART_byte(0xF0);
				NACK = 0x00;
			}
			wake_Flag &= ~RESET;
		}

		if(wake_Flag & READ_OTP)
		{
			unsigned char output_buffer[64];
			int p = 0;
			for (;p<64;p++) {
				output_buffer[p] = 0x00;
			}

			unsigned char x=0;

			unsigned int OTP_ADDRESS = 0xA000;
			I2C_Read_2command(1, I2C_ADDRESS, OTP_ADDRESS, output_buffer);
			if (NACK == 0x01) {
				write_UART_byte(0xF0);
				NACK = 0x00;
			}
			else {
				write_UART_byte(0xC0);
				for (;x<8;x++)
				{
					I2C_Read_2command(64, I2C_ADDRESS, OTP_ADDRESS, output_buffer);

					for (p = 0; p < 64; p++) {
						write_UART_byte(output_buffer[p]);
					}
					OTP_ADDRESS = OTP_ADDRESS + 64;

				}
			}
			wake_Flag &= ~READ_OTP;
		}
		if(wake_Flag & READ_REPORT)
		{
			I2C_output[0] = 0x11;
			I2C_output[1] = 0x02;
			I2C_output[2] = 0x00;
			I2C_output[3] = 0x07;
			I2C_Write_2command_no_stop(4, I2C_ADDRESS, 0x0600, I2C_output, 0);
			if (NACK == 0x01) {
				write_UART_byte(0xF0);
				NACK = 0x00;
			}
			else {

				I2C_Read_no_reg(11, I2C_ADDRESS, I2C_output);
				if (NACK == 0x01) {
					write_UART_byte(0xF0);
					NACK = 0x00;
				}
				else {
					write_UART_byte(0xEC);
					write_UART_byte(11);
					for (i = 0;i <11; i++) {
						write_UART_byte(I2C_output[i]);
					}
				}

			}
			wake_Flag &= ~READ_REPORT;
		}
		if(wake_Flag & READ_INPUT_REPORT)
		{

			I2C_Read_no_reg(11, I2C_ADDRESS, I2C_output);
			if (NACK == 0x01) {
				write_UART_byte(0xF0);
				NACK = 0x00;
			}
			else {
				write_UART_byte(0xEC);
				write_UART_byte(11);
				for (i = 0;i <11; i++) {
					write_UART_byte(I2C_output[i]);
				}
			}
			wake_Flag &= ~READ_INPUT_REPORT;
		}
	}
}

void testfunction(char* array)
{


}
void write_UART_byte(unsigned char byte)
{
	// Wait for UART communication to complete
	while(!(IFG2&UCA0TXIFG));

	// Push byte to send to TX buffer
	UCA0TXBUF = byte;
}

void CLOCK_INIT(void)
{
	BCSCTL2 = SELM_0 + DIVM_0 + DIVS_0;

	if (CALBC1_16MHZ != 0xFF)
	{
		/* Adjust this accordingly to your VCC rise time */
		__delay_cycles(100000);
		// Follow recommended flow. First, clear all DCOx and MODx bits. Then
		// apply new RSELx values. Finally, apply new DCOx and MODx bit values.
		DCOCTL = 0x00;
		BCSCTL1 = CALBC1_16MHZ;      							/* Set DCO to 8MHz */
		DCOCTL = CALDCO_16MHZ;
	}

	BCSCTL1 |= XT2OFF + DIVA_0;
	BCSCTL3 = XT2S_0 + LFXT1S_2 + XCAP_1;
}

void PORT_INIT(void)
{
	// P1.1 = RXD, P1.2=TXD P1.7=SDA P1.6=SCL
	P1SEL 	= BIT1 + BIT2 + BIT6 + BIT7;
	P1SEL2 	= BIT1 + BIT2 + BIT6 + BIT7;

	// Red LED on launchpad set to output P1.4 EN set to output
	P1DIR	 = BIT0;
//	P1IES 	|= BIT4;
//	P1IE 	|= BIT4;
}

void UART_I2C_INIT(void)
{
	// SMCLK
	UCA0CTL1 |= UCSSEL_2;

	// Set Bitrate registers 8MHz / 0x0341 = 9603 bits/second
	UCA0BR0 = 0x83;
	UCA0BR1 = 0x06;

	// Modulation UCBRSx = 5
	UCA0MCTL = UCBRS2 + UCBRS0;

	// **Initialize USCI state machine**
	UCA0CTL1 &= ~UCSWRST;


	//Set UCSWRST to configure I2C bus
	UCB0CTL1 |= UCSWRST;

	//I2C, synchronous master mode
	UCB0CTL0 = UCMST + UCMODE_3 + UCSYNC;

	//Use 8MHz SMCLK
	UCB0CTL1 = UCSSEL_2 + UCSWRST;

	//Set Bitrate register 8MHz/20 = 400kHz
	UCB0BR0 = 16;

	//Reset UCSWRST to enable I2C communication
	UCB0CTL1 &= ~UCSWRST;

	// Enable USCI_A0 RX interrupt
	IE2 |= UCA0RXIE;
}
void FLASH_INIT(void)
{
	 FCTL2 = FWKEY + FSSEL_1 + FN4 + FN2 + FN0;
}
/*******************************************************
 * 		   Interrupt service routine for UART          *
 *-----------------------------------------------------*
 * Decodes serial flags and stores the resulting bytes *
 * into the array input_buffer[].					   *
 * 													   *
 * Also determines if the I2C transmission NACKed due  *
 * to a not acknowledge bit detection				   *
 *******************************************************/
#pragma vector=USCIAB0RX_VECTOR
__interrupt void USCI0RX_ISR(void)
{
	//Interrupt flagged because of UART event
	if(!(UCB0STAT & UCNACKIFG))
	{
		// store input byte from UART receive buffer
		unsigned char in_Byte = UCA0RXBUF;

		// UART flag detection
		if(FLAG & 0x01)
		{
			switch (in_Byte)
			{
				case UART_RESET_FLAG: 		wake_Flag |= RESET;
											_bic_SR_register_on_exit(LPM0_bits);
											break;


				case UART_READ_OTP_FLAG:	wake_Flag |= READ_OTP;
											_bic_SR_register_on_exit(LPM0_bits);
											break;


				case UART_READ_REPORT_FLAG: wake_Flag |= READ_REPORT;
											_bic_SR_register_on_exit(LPM0_bits);
											break;

				case UART_READ_INPUT_REPORT_FLAG: wake_Flag |= READ_INPUT_REPORT;
											_bic_SR_register_on_exit(LPM0_bits);
											break;

				case UART_WAKE_FLAG: 		wake_Flag |= WAKE;
											_bic_SR_register_on_exit(LPM0_bits);
											break;


				case UART_SLEEP_FLAG: 		wake_Flag |= SLEEP;
											_bic_SR_register_on_exit(LPM0_bits);
											break;


				case UART_LED_FLAG: 		FLAG = 0x02;
											counter = 1;
											input_reg_address[0] = in_Byte;
											byte_count = 0;
											break;

				case UART_I2C_ADDRESS_FLAG: FLAG = 0x02;
											counter = 1;
											input_reg_address[0] = in_Byte;

											byte_count = 0;
											break;

				case UART_OTP_DATA_INC: 	first =1;
											//block_count = NUM_BLOCKS;
											//byte_count = BLOCK_SIZE;
											FLAG = 0;											//Flag found
											input_buffer[0] = in_Byte;							//Store flag byte
											counter = 0;										//counter for input_buffer array
											break;


				case UART_OTP_WRITE_SETUP: 	FLAG = 0x02;
											counter = 1;
											input_reg_address[0] = in_Byte;
											byte_count = 3;
											break;

				default: break;
			}
		}
		else if(FLAG & 0x02)
		{
			if(byte_count)
			{
				input_reg_address[counter] = in_Byte;
				counter++;
				byte_count--;
			}
			else
			{
				input_reg_address[counter] = in_Byte;
				FLAG = 1;

				if(input_reg_address[0] == UART_LED_FLAG)
				{
					wake_Flag |= LED;
					_bic_SR_register_on_exit(LPM0_bits);
				}
				else if(input_reg_address[0] == UART_I2C_ADDRESS_FLAG)
				{
					wake_Flag |= ADDRESS;
					_bic_SR_register_on_exit(LPM0_bits);
				}
				else if(input_reg_address[0] == UART_OTP_WRITE_SETUP)
				{

					OTP_start_add = input_reg_address[2] << 8;
					OTP_start_add += input_reg_address[1];				//Start address of OTP

					OTP_count = input_reg_address[4] << 8;
					OTP_count += input_reg_address[3];					//How many bytes to program

					RAM_ADDRESS = 0xA000 + OTP_start_add;				//Sram location to write to
					OTP_count1 = OTP_count;



					byte_count = 1;
					while(!(IFG2&UCA0TXIFG));

					// Push byte to send to TX buffer
					UCA0TXBUF = 0xED;
					while(!(IFG2&UCA0TXIFG));

				}
			}
		}
		else															//Fill input_buffer[]
		{
			if(byte_count < OTP_count)
			{
				if(byte_count % 0x40 != 0)
				{
					input_buffer[counter] = in_Byte;
					counter++;
					byte_count++;
				}
				else
				{
					input_buffer[counter] = in_Byte;
					if(first)
					{
						ptr = (char *) FLASH_PTR;
						FCTL1 = FWKEY + ERASE;              // Set Erase bit
						FCTL3 = FWKEY;                      // Clear Lock bit
						*ptr = 0;                           // Dummy write to erase Flash segment
						FCTL1 = FWKEY + WRT;	            // Set WRT bit for write operation
						first=0;
					}
					unsigned char x = 0;
					for(x=0;x<64;x++)
					{
						*ptr++ = input_buffer[x];
					}

					byte_count++;
					counter = 0;

					while(!(IFG2&UCA0TXIFG));

					// Push byte to send to TX buffer
					UCA0TXBUF = 0xED;
					while(!(IFG2&UCA0TXIFG));

				}
			}
			else
			{

					input_buffer[counter] = in_Byte;
					if(first)
					{
						ptr = (char *) FLASH_PTR;
						FCTL1 = FWKEY + ERASE;              // Set Erase bit
						FCTL3 = FWKEY;                      // Clear Lock bit
						*ptr = 0;                           // Dummy write to erase Flash segment
						FCTL1 = FWKEY + WRT;	            // Set WRT bit for write operation
						first=0;
					}
					unsigned char x = 0;
					for(x=0;x<=counter;x++)
					{
						*ptr++ = input_buffer[x];
					}
					FCTL1 = FWKEY;                      	// Clear WRT bit
					FCTL3 = FWKEY + LOCK;               	// Set LOCK bit
					//block_count--;
					//byte_count = BLOCK_SIZE;


					FLAG = 1;

					write_UART_byte(0xDD);


					wake_Flag |= 0x01;
					// Wake CPU
					_bic_SR_register_on_exit(LPM0_bits);

			}
		}
	}

	// NACK detected
	else
	{
		// Set global variable for NACK
		NACK=1;

		// Clear NACK interrupt flag
		UCB0STAT &= ~UCNACKIFG;

		// wake CPU
		_bic_SR_register_on_exit(LPM0_bits);
	}


}


/*******************************************************
 * 		   Interrupt service routine for PORT1         *
 *-----------------------------------------------------*
 * Runs when /INT of the TCA7313 asserts LOW. Flags	   *
 * MCU to decode which input on the TCA7313 changed.   *
 *******************************************************/
#pragma vector=PORT1_VECTOR
__interrupt void PORT1_ISR_HOOK(void)
{


	_bic_SR_register_on_exit(LPM0_bits);
}

void print_UART(char* string)
{
	while(*string)
	{
		write_UART_byte(*string++);
	}
}



/*******************************************************
 * 		   Interrupt service routine for I2C           *
 *-----------------------------------------------------*
 * These control the I2C read and write functions	   *
 * located within USCI_I2C.c                           *
 *******************************************************/
#pragma vector=USCIAB0TX_VECTOR
__interrupt void USCI0TX_ISR_HOOK(void)
{
	unsigned short lowPowerBits = 0;

    if (IFG2 & UCB0TXIFG) {
		/* USCI_B0 Transmit Interrupt Handler */
		lowPowerBits = USCIAB0TX_ISR();

		/* Exit low power mode based on return bits */
		_bic_SR_register_on_exit(lowPowerBits);
    }
    else {
		/* USCI_B0 Receive Interrupt Handler */
		USCIAB0RX_ISR();

		/* Enter active mode on exit */
		_bic_SR_register_on_exit(LPM4_bits);
    }
}




